import sqlite3
import csv
import gzip
import alive_progress
from timeit import default_timer as timer
from humanfriendly import format_timespan, format_size
import os
import sys

def getCordinate(value):
    return [int(value.split(",")[0]), int(value.split(",")[1])]


# config
database_file = 'data/data.db'
rawdata_path = 'data/raw'

# create list for file locations
file_list = []
for i in range(0, 79):
    number = str(i).zfill(12)
    path = f'{rawdata_path}/2022_place_canvas_history-{number}.csv.gz'
    if os.path.exists(path):
        file_list.append(path)
    else:
        path = f'{rawdata_path}/2022_place_canvas_history-{number}.csv.gzip'
        if os.path.exists(path):
            file_list.append(path)

# confirm
print(f"creating database at '{database_file}' using the following files:")
for file in file_list:
    print(file)
print(f"\n")

if os.path.exists(database_file):
    print(f"WARNING! The 'place' table in '{database_file}' will be overwritten!\n")
print("This process can take up to an hour... or longer")
shall = input(
    f"Are you sure you want to continue?\n(y/N) ").lower() == 'y'

if shall:
    print(f"\nThis may take a while...\nEach file will be committed to the database individually.\n")

    # connect to database
    con = sqlite3.connect(database_file)
    cur = con.cursor()

    # drop table
    cur.execute('''DROP TABLE IF EXISTS place''')

    # create table
    cur.execute('''CREATE TABLE place (color text, posx number, posy number, user text, time timestamp)''')
    con.commit()
    database_filesize = os.path.getsize(database_file)

    # process data
    start_time = timer()
    total_line_count = 0
    file_count = 0
    eta = 0
    size_estimate = 0
    for file in file_list:
        file_count += 1

        with gzip.open(file, 'rt') as csv_file:
            # read csv
            csv_reader = csv.reader(csv_file, delimiter=',')

            csv_list = None
            file_length = 0

            with alive_progress.alive_bar(title=f"opening {file}\t", dual_line=True, monitor=False, stats=False) as bar:
                if file_count > 1:
                    bar.text = f'Database file size: {format_size(database_filesize)} | Final size estimate: {format_size(size_estimate)} | eta: {format_timespan(max(eta - (timer() - start_time), 0))}'

                csv_list = list(csv_reader)
                file_length = len(csv_list)
                bar(1)

            line_count = 0
            with alive_progress.alive_bar(file_length, dual_line=True, title=f"processing {file}\t") as bar:
                if file_count > 1:
                    bar.text = f'Database file size: {format_size(database_filesize)} | Final size estimate: {format_size(size_estimate)} | eta: {format_timespan(max(eta - (timer() - start_time), 0))}'

                for row in csv_list:
                    if line_count == 0:
                        line_count += 1
                    else:
                        pos = getCordinate(row[3])

                        # insert
                        cur.execute(f"INSERT INTO place VALUES ('{row[2]}', '{pos[0]}', '{pos[1]}', '{row[1]}', '{ row[0]}')")

                        line_count += 1
                        total_line_count += 1

                    bar()

        # Save (commit) the changes
        con.commit()

        # read file size
        database_filesize = os.path.getsize(database_file)

        # calculate estimate
        eta = ((timer() - start_time) / file_count) * len(file_list)
        size_estimate = (database_filesize / file_count) * len(file_list)

    # create index
    print("\n")
    with alive_progress.alive_bar(title="creating index... this may take a while") as bar:
        cur.execute('CREATE INDEX `timestamp_index` ON `place` (`time`);')
        bar()

    # show stats
    end_time = timer()
    print(
        f"\nprocessed {total_line_count} placed tiles from {len(file_list)} file/s, including index in {format_timespan(end_time - start_time)}")
    print(f'Database file size: {format_size(os.path.getsize(database_file))}')

    # Save (commit) the changes
    con.commit()

    # close connection
    con.close()
else:
    print("aborting.")
